/**********************************************************************
 * SQLColAttribute
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 **********************************************************************/

#include "driver.h"

SQLRETURN SQLColAttribute(	SQLHSTMT        hDrvStmt,
							SQLUSMALLINT    nCol,
							SQLUSMALLINT    nFieldIdentifier,
							SQLPOINTER      pszValue,
							SQLSMALLINT     nValueLengthMax,
							SQLSMALLINT     *pnValueLength,
							SQLPOINTER      pnValue )
{
    HDRVSTMT	hStmt	= (HDRVSTMT)hDrvStmt;
	HTXTCOLUMN	pColumn;
	int			nValue = 0;

    /* SANITY CHECKS */
    if( !hStmt )
        return SQL_INVALID_HANDLE;

	if ( !hStmt->hTable )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR No result set." );
		return SQL_ERROR;
	}

	if ( nCol < 1 || nCol > hStmt->hTable->hColumns->nItems )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR Invalid column" );
		return SQL_ERROR;
	}

    /* OK */
    lstGoto( hStmt->hTable->hColumns, nCol-1 );
    /* lstGet() will recurse down to the base table and return a ref to it */
    pColumn = (HTXTCOLUMN)lstGet( hStmt->hTable->hColumns );

    switch( nFieldIdentifier )
    {
	case SQL_DESC_AUTO_UNIQUE_VALUE:
		nValue = 0;
		break;
	case SQL_DESC_BASE_COLUMN_NAME:
		strncpy( pszValue, pColumn->pszName, nValueLengthMax );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_BASE_TABLE_NAME:
		strncpy( pszValue, ((HTXTTABLE)pColumn->hTable)->pszName, nValueLengthMax );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_CASE_SENSITIVE:
		nValue = 1;
		break;
	case SQL_DESC_CATALOG_NAME:
		strncpy( pszValue, ((HTXTDATABASE)((HTXTTABLE)pColumn->hTable)->hDatabase)->pszName, nValueLengthMax );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
    case SQL_DESC_CONCISE_TYPE:
		nValue = pColumn->nSQLType;
		break;
	case SQL_DESC_COUNT:
		nValue = hStmt->hTable->hColumns->nItems;
		break;
	case SQL_DESC_DISPLAY_SIZE:
		nValue = pColumn->nLength;
		break;
	case SQL_DESC_FIXED_PREC_SCALE:
		nValue = pColumn->nPrecision;
		break;
	case SQL_DESC_LABEL:
		strncpy( pszValue, pColumn->pszName, nValueLengthMax );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_LENGTH:
		nValue = pColumn->nLength;
		break;
	case SQL_DESC_LITERAL_PREFIX:
		pszValue = (char*)strdup( "" );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_LITERAL_SUFFIX:
		pszValue = (char*)strdup( "" );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_LOCAL_TYPE_NAME:
		pszValue = (char*)strdup( "" );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_NAME:
		strncpy( pszValue, pColumn->pszName, nValueLengthMax );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_NULLABLE:
		nValue = SQL_NO_NULLS;
		break;
	case SQL_DESC_NUM_PREC_RADIX:
		nValue = 0;
		break;
	case SQL_DESC_OCTET_LENGTH:
		nValue = pColumn->nLength;
		break;
	case SQL_DESC_PRECISION:
		nValue = pColumn->nPrecision;
		break;
	case SQL_DESC_SCALE:
		nValue = pColumn->nPrecision;
		break;
	case SQL_DESC_SCHEMA_NAME:
		strncpy( pszValue, ((HTXTDATABASE)((HTXTTABLE)pColumn->hTable)->hDatabase)->pszName, nValueLengthMax );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_SEARCHABLE:
		nValue = SQL_PRED_SEARCHABLE;
		break;
	case SQL_DESC_TABLE_NAME:
		strncpy( pszValue, ((HTXTTABLE)pColumn->hTable)->pszName, nValueLengthMax );
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_TYPE:
		nValue = pColumn->nSQLType;
		break;
    case SQL_DESC_TYPE_NAME:
        switch ( pColumn->nSQLType )
        {
        case SQL_NUMERIC:
            pszValue = (char*)strdup( "NUMERIC" );
            break;
        case SQL_DATETIME:
            pszValue = (char*)strdup( "DATETIME" );
            break;
        case SQL_VARCHAR:
        default:
            pszValue = (char*)strdup( "VARCHAR" );
        }
		if ( pnValueLength )
            *pnValueLength = strlen( pszValue );
		break;
	case SQL_DESC_UNNAMED:
		nValue = SQL_NAMED;
		break;
	case SQL_DESC_UNSIGNED:
		nValue = 0;
		break;
	case SQL_DESC_UPDATABLE:
		nValue = SQL_ATTR_READONLY;
		break;
    default:
			sprintf( hStmt->szSqlMsg, "Invalid nFieldIdentifier value of %d", nFieldIdentifier );
			logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );
            return SQL_ERROR;
    }

	if ( pnValue )
		*(int*)pnValue = nValue;

    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_SUCCESS" );

    return SQL_SUCCESS;
}



