/********************************************************************
 * SQLColumns
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 ********************************************************************/

#include "driver.h"

SQLRETURN SQLColumns( 	SQLHSTMT    hDrvStmt,
						SQLCHAR     *szCatalogName,
						SQLSMALLINT nCatalogNameLength,
						SQLCHAR     *szSchemaName,
						SQLSMALLINT nSchemaNameLength,
						SQLCHAR     *szTableName,
						SQLSMALLINT nTableNameLength,
						SQLCHAR     *szColumnName,
						SQLSMALLINT nColumnNameLength )
{
    HDRVSTMT	    hStmt		= (HDRVSTMT)hDrvStmt;
    HTXTDATABASE    hDatabase       = 0;
    HTXTTABLE       hTable          = 0;
    HTXTTABLE       hSourceTable    = 0;
	HTXTCOLUMN	    hColumn         = 0;
	char **         ppRow;
    char		    szBuffer[101];



    /* SANITY CHECKS */
    if( !hStmt )
        return SQL_INVALID_HANDLE;

	sprintf( hStmt->szSqlMsg, "hStmt = $%08lX", hStmt );
    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );

	if ( szTableName == NULL || szTableName[0] == '\0' )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR Must supply a valid table name" );
		return SQL_ERROR;
	}

    /**************************
     * get current database
     * - remember, hDbcExtras->hDatabases is actually a list with just one item 
     * this allows a ref to be kept in LST
     **************************/
    hDatabase = ((HDRVDBC)hStmt->hDbc)->hDatabase;
    if ( !hDatabase )
    {
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR Not connected to database." );
		return SQL_ERROR;
    }

    /**************************
	 * close any existing result?
     **************************/
	if ( hStmt->hTable )
    {
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "Statement already being used." );
		return SQL_ERROR;
    }

	/************************
	 * LETS MANUALLY CREATE A RESULT TABLE
     * - we only support returning columns on a specific table so the result will 
     * have only one row
	 ************************/
    
    /* TABLE */
    hTable = (HTXTTABLE)malloc( sizeof(TXTTABLE) );
    lstOpen( &(hTable->hColumns) ); 
    lstSetFreeFunc( hTable->hColumns, (void(*)(void*))TXTFreeColumn );
    lstOpen( &(hTable->hRows) );
    lstSetFreeFunc( hTable->hRows, (void(*)(void*))TXTFreeRow );
    hTable->pszDescription = (char*)strdup( szTableName );
    hTable->pszName = (char*)strdup( "SQLColumns" );
    hTable->hDatabase = hDatabase;
    hStmt->hTable = hTable;

    /* TABLE COLUMNS */
    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_CAT" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_SCHEM" );
    lstAppend( hTable->hColumns, hColumn );
	
    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_NAME" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "COLUMN_NAME" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "DATA_TYPE" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TYPE_NAME" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "COLUMN_SIZE" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "BUFFER_LENGTH" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "NUM_PREC_RADIX" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "DECIMAL_DIGITS" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "NULLABLE" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "REMARKS" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "COLUMN_DEF" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "SQL_DATA_TYPE" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "SQL_DATETIME_SUB" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "CHAR_OCTET_LENGTH" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "ORDINAL_POSITION" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "IS_NULLABLE" );
    lstAppend( hTable->hColumns, hColumn );

    /* find table (szTableName) in cache */
    lstFirst( hDatabase->hTables );
    while ( !lstEOL( hDatabase->hTables ) )
    {
        hSourceTable = (HTXTTABLE)lstGet( hDatabase->hTables );
        if ( strcmp( hSourceTable->pszName, szTableName ) == 0 )
        {
            /* ROWS (just one row) */
        	lstFirst( hSourceTable->hColumns );
        	while ( !lstEOL( hSourceTable->hColumns ) )
        	{
        		ppRow		= malloc( sizeof(char*) * hTable->hColumns->nItems );
        		hColumn 	= (HTXTCOLUMN)lstGet( hSourceTable->hColumns );
        		ppRow[0]	= (char *)strdup( hDatabase->pszName );
        		ppRow[1]	= (char *)strdup( hDatabase->pszName );
        		ppRow[2]	= (char *)strdup( hSourceTable->pszName );
        		ppRow[3]	= (char *)strdup( hColumn->pszName );
        		ppRow[4]	= (char *)strdup( "" );
                switch ( hColumn->nSQLType )
                {
                case SQL_NUMERIC:
                    ppRow[5] = (char*)strdup( "NUMERIC" );
                    break;
                case SQL_DATETIME:
                    ppRow[5] = (char*)strdup( "DATETIME" );
                    break;
                case SQL_VARCHAR:
                default:
                    ppRow[5] = (char*)strdup( "VARCHAR" );
                }
        		sprintf( szBuffer, "%d", hColumn->nLength );
        		ppRow[6]	= (char *)strdup( szBuffer );
        		ppRow[7]	= (char *)strdup( "" );
        		ppRow[8]	= (char *)strdup( "" );
        		ppRow[9]	= (char *)strdup( "" );
        		ppRow[10]	= (char *)strdup( "0" );
        		ppRow[11]	= (char *)strdup( "" );
        		ppRow[12]	= (char *)strdup( "" );
        		ppRow[13]	= (char *)strdup( "" );
        		ppRow[14]	= (char *)strdup( "" );
        		ppRow[15]	= (char *)strdup( "" );
        		ppRow[16]	= (char *)strdup( "" );
        		ppRow[17]	= (char *)strdup( "" );
        
        		lstAppend( hTable->hRows, ppRow );
        
        		lstNext( hSourceTable->hColumns );
        	}
            break;
        }
    
        lstNext( hDatabase->hTables );
    }

    /* wrap it up */
    hStmt->nRowsAffected = hTable->hRows->nItems;

    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "SQL_SUCCESS" );
    return SQL_SUCCESS;
}



