/**********************************************************************
 * SQLConnect
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 **********************************************************************/

#include "driver.h"

int cbDatabaseSearch_( HLST h, void *pData );


SQLRETURN SQLConnect(	SQLHDBC        hDrvDbc,
						SQLCHAR        *szDataSource,
						SQLSMALLINT    nDataSourceLength,
						SQLCHAR        *szUID,
						SQLSMALLINT    nUIDLength,
						SQLCHAR        *szPWD,
						SQLSMALLINT    nPWDLength
                          )
{
	HDRVDBC 	hDbc	= (HDRVDBC)hDrvDbc;
    char    	szDATABASE[INI_MAX_PROPERTY_VALUE+1];
    char    	szTRACE[INI_MAX_PROPERTY_VALUE+1];
    char    	szTRACEFILE[INI_MAX_PROPERTY_VALUE+1];
    HLST        hDatabases;
    HINI        hDatabase;

    /* SANITY CHECKS */
    if( SQL_NULL_HDBC == hDbc )
		return SQL_INVALID_HANDLE;

	sprintf( hDbc->szSqlMsg, "hDbc=$%08lX szDataSource=(%s)", hDbc, szDataSource );
	logPushMsg( hDbc->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hDbc->szSqlMsg );

    if( hDbc->bConnected == 1 )
    {
		logPushMsg( hDbc->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR Already connected" );
        return SQL_ERROR;
    }

    if ( strlen( szDataSource ) > ODBC_FILENAME_MAX+INI_MAX_OBJECT_NAME )
    {
		logPushMsg( hDbc->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR Given Data Source is too long. I consider it suspect." );
        return SQL_ERROR;
    }

    /********************
     * gather and use any required DSN properties
	 * - DATABASE
     ********************/
    szDATABASE[0] 		= '\0';
    szTRACE[0] 			= '\0';
    szTRACEFILE[0] 		= '\0';
	SQLGetPrivateProfileString( szDataSource, "DATABASE", "", szDATABASE, sizeof(szDATABASE), ".odbc.ini" );
	if ( szDATABASE[0] == '\0' )
	{
		sprintf( hDbc->szSqlMsg, "SQL_ERROR Could not find Database entry for %s in system information", szDataSource );
		logPushMsg( hDbc->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hDbc->szSqlMsg );
		return SQL_ERROR;
	}
	SQLGetPrivateProfileString( szDataSource, "TRACE", "", szTRACE, sizeof(szTRACE), ".odbc.ini" );
	SQLGetPrivateProfileString( szDataSource, "TRACEFILE", "", szTRACEFILE, sizeof(szTRACEFILE), ".odbc.ini" );

    /********************
     * SET TRACING ON HERE
	 * - SQLDisconnect WILL TURN IT OFF (by freeing pszLogFile)
	 * - Subordinate handles (ie HSTMT) will inherit the trace
     ********************/
	if ( szTRACE )
	{
		if ( szTRACE[0] == '1' || toupper(szTRACE[0]) == 'Y' || toupper(szTRACE[0]) == 'O' )
		{
			if ( szTRACEFILE )
			{
				hDbc->hLog->pszLogFile = (char*)strdup( szTRACEFILE );
			}
		}
	}

    /********************
     * LOOK FOR DATABASE IN CACHE
     * - I may search for it twice. The first time is obvious. If the first search fails
     * then I attempt to load it. I then search for it again despite having the handle
     * available. I do this because I want an open cursor containing the database. This 
     * will create a ref to the database in the cache which will ensure that the database 
     * will stay loaded, at least until we are done... perhaps longer if there are other 
     * refs.
     ********************/
    hDatabases = lstOpenCursor( hCache->hDatabases, cbDatabaseSearch_, szDATABASE );
    if ( hDatabases->nItems < 1 )
    {
        lstClose( hDatabases );
        TXTLoadDatabase( hCache, szDATABASE );
        /* NOW ENSURE WE HAVE A REF TO THE DB */
        hDatabases = lstOpenCursor( hCache->hDatabases, cbDatabaseSearch_, szDATABASE );
        if ( hDatabases->nItems < 1 )
        {
            logPushMsg( hDbc->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "Failed to load database." );
            return SQL_ERROR;
        }
    }
    
    hCache->hDatabases->pExtras = 0;
    hDbc->bConnected = 1;
    hDbc->hDatabase = lstGet( hDatabases );
    hDbc->hDatabases = hDatabases; /* CLOSE THIS CURSOR WHEN WE ARE DONE WITH THE DB */

    logPushMsg( hDbc->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "SQL_SUCCESS" );

    return SQL_SUCCESS;
}

/********************
 * CALLBACK FUNC USED TO SEARCH FOR DATABASE IN CACHE
 ********************/
int cbDatabaseSearch_( HLST h, void *pData )
{
    HTXTDATABASE hDatabase = (HTXTDATABASE)pData;

    if ( strcmp( h->pExtras, hDatabase->pszName ) )
        return 1;
    else if ( strcmp( h->pExtras, hDatabase->pszFileName ) )
        return 1;

    return 0;
}


