/**********************************************************************
 * SQLDescribeCol
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 **********************************************************************/

#include "driver.h"

SQLRETURN SQLDescribeCol( SQLHSTMT    	hDrvStmt,
						  SQLUSMALLINT	nCol,
						  SQLCHAR     	*szColName,
						  SQLSMALLINT 	nColNameMax,
						  SQLSMALLINT 	*pnColNameLength,
						  SQLSMALLINT 	*pnSQLDataType,
						  SQLUINTEGER 	*pnColSize,
						  SQLSMALLINT 	*pnDecDigits,
						  SQLSMALLINT 	*pnNullable )
{
    HDRVSTMT 	hStmt			= (HDRVSTMT)hDrvStmt;
	HTXTCOLUMN	hColumn;

	/* SANITY CHECKS */
	if ( !hStmt )
		return SQL_INVALID_HANDLE;

	sprintf( hStmt->szSqlMsg, "hStmt = $%08lX", hStmt );
	logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );

	if ( !hStmt->hTable )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR No result set." );
		return SQL_ERROR;
	}

	if ( nCol < 1 || nCol > hStmt->hTable->hColumns->nItems )
	{
		sprintf( hStmt->szSqlMsg, "SQL_ERROR Column %d is out of range. Range is 1 - %d", nCol, hStmt->hTable->hColumns->nItems );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );
		return SQL_ERROR;
	}

	/****************************
	 * Ok
	 ***************************/
	lstGoto( hStmt->hTable->hColumns, nCol );
	hColumn = (HTXTCOLUMN)lstGet( hStmt->hTable->hColumns );

	if ( szColName )
		strncpy( szColName, hColumn->pszName, nColNameMax );
	if ( pnColNameLength )
		*pnColNameLength = strlen( szColName );
	if ( pnSQLDataType )
		*pnSQLDataType = hColumn->nSQLType;
	if ( pnColSize )
		*pnColSize = hColumn->nLength;
	if ( pnDecDigits )
		*pnDecDigits = hColumn->nPrecision;
	if ( pnNullable )
		*pnNullable = 1;

	logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "SQL_SUCCESS" );
	return SQL_SUCCESS;
}



