/**********************************************************************
 * SQLFreeStmt
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 **********************************************************************/

#include "driver.h"

SQLRETURN SQLFreeStmt(	SQLHSTMT        hDrvStmt,
						SQLUSMALLINT    nOption )
{
    return SQLFreeStmt_( hDrvStmt, nOption );
}


SQLRETURN SQLFreeStmt_(	SQLHSTMT        hDrvStmt,
						SQLUSMALLINT    nOption )
{
    HDRVSTMT hStmt	= (HDRVSTMT)hDrvStmt;

	/* SANITY CHECKS */
    if( hStmt == SQL_NULL_HSTMT )
        return SQL_INVALID_HANDLE;

	sprintf( hStmt->szSqlMsg, "hStmt (%d)", hStmt );
    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );

    /*********
     * RESET PARAMS
     *********/
    switch( nOption )
    {
	case SQL_CLOSE:
		break;

    case SQL_DROP:
        return DRVFreeStmt( hStmt );

	case SQL_UNBIND:
		break;

    case SQL_RESET_PARAMS:
        break;

    default:
		sprintf( hStmt->szSqlMsg, "SQL_ERROR Invalid nOption=%d", nOption );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );
        return SQL_ERROR;
    }

    return SQL_SUCCESS;
}


SQLRETURN DRVFreeStmt( HDRVSTMT hStmt )
{
	HDRVSTMT	hPrevStmt;
	SQLRETURN	nReturn;

	if ( hStmt == SQL_NULL_HDBC )
		return SQL_ERROR;

	/* SPECIAL CHECK FOR FIRST IN LIST 				*/
	if ( ((HDRVDBC)hStmt->hDbc)->hFirstStmt == hStmt )
		((HDRVDBC)hStmt->hDbc)->hFirstStmt = hStmt->pNext;
	
	/* SPECIAL CHECK FOR LAST IN LIST 				*/
	if ( ((HDRVDBC)hStmt->hDbc)->hLastStmt == hStmt )
		((HDRVDBC)hStmt->hDbc)->hLastStmt = hStmt->pPrev;

	/* EXTRACT SELF FROM LIST						*/
	if ( hStmt->pPrev != SQL_NULL_HSTMT )
		hStmt->pPrev->pNext = hStmt->pNext;
	if ( hStmt->pNext != SQL_NULL_HSTMT )
		hStmt->pNext->pPrev = hStmt->pPrev;

	/* FREE STANDARD MEMORY */
	if ( hStmt->pszQuery ) free( hStmt->pszQuery );

    lstClose( hStmt->hBoundCols );
    lstClose( hStmt->hTable );

	logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "SQL_SUCCESS" );
	logClose( hStmt->hLog );
	
	free( hStmt );

	return SQL_SUCCESS;
}


SQLRETURN DRVFreeStmtList( HDRVDBC hDbc )
{

	if ( hDbc == SQL_NULL_HDBC )
		return SQL_SUCCESS;

	if ( hDbc->hFirstStmt == NULL )
		return SQL_SUCCESS;

	while ( DRVFreeStmt( hDbc->hFirstStmt ) == SQL_SUCCESS )
	{
	}

	return SQL_SUCCESS;
}



