/**********************************************************************
 * SQLGetData
 *
 * 1. MiniSQL server sends all data as ascii strings so things are
 * simplified. We always convert from string to nTargetType.
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 **********************************************************************/

#include "driver.h"

SQLRETURN SQLGetData( SQLHSTMT      hDrvStmt,
					  SQLUSMALLINT  nCol,						/* 1 based */
					  SQLSMALLINT   nTargetType,				/* C DATA TYPE */
					  SQLPOINTER    pTarget,
					  SQLINTEGER    nTargetLength,
					  SQLINTEGER    *pnLengthOrIndicator )
{
	return SQLGetData_(	hDrvStmt,
						nCol-1,
						nTargetType,		
						pTarget,
						nTargetLength,
						pnLengthOrIndicator );
}


SQLRETURN SQLGetData_(	SQLHSTMT      hDrvStmt,
						SQLUSMALLINT  nCol,						/* zero based */
						SQLSMALLINT   nTargetType,				/* C DATA TYPE */
						SQLPOINTER    pTarget,
						SQLINTEGER    nTargetLength,
						SQLINTEGER    *pnLengthOrIndicator )
{
    HDRVSTMT	hStmt			= (HDRVSTMT)hDrvStmt;
	HTXTCOLUMN	hColumn;
	char		**ppRow         = NULL;
	char    	*pSourceData    = NULL;

	/* SANITY CHECKS */
	if ( hStmt == SQL_NULL_HSTMT )
		return SQL_INVALID_HANDLE;

	if ( !hStmt->hTable )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR No result set." );
		return SQL_ERROR;
	}

	/**********************************************************************
	 * GET pSourceData FOR NORMAL RESULT SETS
	 **********************************************************************/
	if ( lstEOL( hStmt->hTable->hRows ) )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR No current row" );
		return SQL_ERROR;
	}

	/**********************************************************************
	 * XREF COL # IN RESULT SET TO COL # IN ROOT TABLE
	 * This is a performance hit but we are not out to be super fast. - PAH
	 **********************************************************************/
	lstGoto( hStmt->hTable->hColumns, nCol );
	hColumn = (HTXTCOLUMN)lstGet( hStmt->hTable->hColumns );
	ppRow	= (char**)lstGet( hStmt->hTable->hRows );

	pSourceData = ppRow[hColumn->nCol];

	/****************************
	 * ALL cols are stored as SQL_CHAR... bad for storage... good for code
	 * SO no need to determine the source type when translating to destination
	 ***************************/	
	if ( pSourceData == NULL )
	{
		/*********************
		 * Now get the col if value = NULL
		 *********************/
		if ( pnLengthOrIndicator != NULL )
			*pnLengthOrIndicator = SQL_NULL_DATA;

		switch ( nTargetType )
		{
		case SQL_C_LONG:
			memset( pTarget, 0, sizeof(int) );
			break;

		case SQL_C_FLOAT:
			memset( pTarget, 0, sizeof(float) );
			break;

		case SQL_C_CHAR:
			*((char *)pTarget) = '\0';
			break;

		default:
			sprintf( hStmt->szSqlMsg, "SQL_ERROR Unknown target type %d", nTargetType );
			logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );
		}
	}
	else
	{
		/*********************
		 * Now get the col when we have a value
		 *********************/
		switch ( nTargetType )
		{
		case SQL_C_LONG:
			*((int *)pTarget) = atoi(pSourceData);
			if ( NULL != pnLengthOrIndicator )
				*pnLengthOrIndicator = sizeof( int );
			break;

		case SQL_C_FLOAT:
			sscanf( pSourceData, "%g", pTarget );
			if ( NULL != pnLengthOrIndicator )
				*pnLengthOrIndicator = sizeof( float );
			break;

		case SQL_C_CHAR:
			strncpy( pTarget, pSourceData, nTargetLength );
			if ( NULL != pnLengthOrIndicator )
				*pnLengthOrIndicator = strlen(pTarget);
			break;

		default:
			sprintf( hStmt->szSqlMsg, "SQL_ERROR Unknown target type %d", nTargetType );
			logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );
		}
	}

	logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "SQL_SUCCESS" );
	return SQL_SUCCESS;
}


