/********************************************************************
 * SQLStatistics
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 ********************************************************************/

#include "driver.h"


SQLRETURN SQLStatistics(	SQLHSTMT        hDrvStmt,
							SQLCHAR         *szCatalogName,
							SQLSMALLINT     nCatalogNameLength,
							SQLCHAR         *szSchemaName,
							SQLSMALLINT     nSchemaNameLength,
							SQLCHAR         *szTableName,      		/* MUST BE SUPPLIED */
							SQLSMALLINT     nTableNameLength,
							SQLUSMALLINT    nTypeOfIndex,
							SQLUSMALLINT    nReserved    )
{
    HDRVSTMT        hStmt	        = (HDRVSTMT)hDrvStmt;
    HTXTDATABASE    hDatabase       = 0;
    HTXTTABLE       hTable          = 0;
	HTXTCOLUMN	    hColumn         = 0;

	/* SANITY CHECKS */
    if( hStmt == SQL_NULL_HSTMT )
        return SQL_INVALID_HANDLE;

	sprintf( hStmt->szSqlMsg, "hStmt = $%08lX", hStmt );
    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );

	if ( szTableName == NULL || *szTableName == '\0' )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_ERROR, "Valid szTableName required" );
		return SQL_ERROR;
	}

    /**************************
	 * close any existing result
     **************************/
	if ( hStmt->hTable )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_ERROR, "Statement in use." );
		return SQL_ERROR;
	}


    /**************************
     * get current database
     **************************/
    hDatabase = ((HDRVDBC)hStmt->hDbc)->hDatabase;

    /**************************
	 * close any existing result?
     **************************/
	if ( hStmt->hTable )
    {
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "Statement already being used." );
		return SQL_ERROR;
    }

	/************************
	 * LETS MANUALLY CREATE A RESULT TABLE
     * - we only support returning columns on a specific table so the result will 
     * have only one row
	 ************************/
    
    /* TABLE */
    hTable = (HTXTTABLE)malloc( sizeof(TXTTABLE) );
    lstOpen( &(hTable->hColumns) );
    lstSetFreeFunc( hTable->hColumns, (void (*)(void*))TXTFreeColumn );
    lstOpen( &(hTable->hRows) );
    lstSetFreeFunc( hTable->hRows, (void (*)(void*))TXTFreeRow );
    hTable->pszDescription = (char*)strdup( szTableName );
    hTable->pszName = (char*)strdup( "SQLStatistics" );
    hTable->hDatabase = hDatabase;
    hStmt->hTable = hTable;

    /* TABLE COLUMNS */
    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_CAT" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_SCHEM" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_NAME" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "NON_UNIQUE" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "INDEX_QUALIFIER" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "INDEX_NAME" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TYPE" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "ORDINAL_POSITION" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "COLUMN_NAME" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "ASC_OR_DESC" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "CARDINALITY" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "PAGES" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "FILTER_CONDITION" );
    lstAppend( hTable->hColumns, hColumn );

    /* ADD ROWS HERE */

    /* wrap it up */
    hStmt->nRowsAffected = hTable->hRows->nItems;

    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "SQL_SUCCESS" );
	return SQL_SUCCESS;
}


