/**********************************************************************
 * SQLTables
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 **********************************************************************/

#include "driver.h"

SQLRETURN SQLTables(  	SQLHSTMT    hDrvStmt,
						SQLCHAR     *szCatalogName,
						SQLSMALLINT nCatalogNameLength,
						SQLCHAR     *szSchemaName,
						SQLSMALLINT nSchemaNameLength,
						SQLCHAR     *szTableName,
						SQLSMALLINT nTableNameLength,
						SQLCHAR     *szTableType,
						SQLSMALLINT nTableTypeLength )
{
    HDRVSTMT	    hStmt		= (HDRVSTMT)hDrvStmt;
    HTXTDATABASE    hDatabase       = 0;
    HTXTTABLE       hTable          = 0;
    HTXTTABLE       hSourceTable    = 0;
	HTXTCOLUMN	    hColumn         = 0;
	char **         ppRow;
    char		    szBuffer[101];



    /* SANITY CHECKS */
    if( !hStmt )
        return SQL_INVALID_HANDLE;

	sprintf( hStmt->szSqlMsg, "hStmt = $%08lX", hStmt );
    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );

    /**************************
     * get current database
     **************************/
    hDatabase = ((HDRVDBC)hStmt->hDbc)->hDatabase;

    /**************************
	 * close any existing result?
     **************************/
	if ( hStmt->hTable )
    {
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "Statement already being used." );
		return SQL_ERROR;
    }

	/************************
	 * LETS MANUALLY CREATE A RESULT TABLE
     * - we only support listing all tables
	 ************************/
    
    /* TABLE */
    hTable = (HTXTTABLE)malloc( sizeof(TXTTABLE) );
    lstOpen( &(hTable->hColumns) );
    lstSetFreeFunc( hTable->hColumns, (void (*)(void*))TXTFreeColumn );
    lstOpen( &(hTable->hRows) );
    lstSetFreeFunc( hTable->hRows, (void (*)(void*))TXTFreeRow );
    hTable->pszDescription = (char*)strdup( szTableName );
    hTable->pszName = (char*)strdup( "SQLTables" );
    hTable->hDatabase = hDatabase;
    hStmt->hTable = hTable;

    /* TABLE COLUMNS */
    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_CAT" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_SCHEM" );
    lstAppend( hTable->hColumns, hColumn );
	
    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_NAME" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "TABLE_TYPE" );
    lstAppend( hTable->hColumns, hColumn );

    hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
    hColumn->hTable = hTable;
    hColumn->nLength = 50;
    hColumn->nPrecision = 0;
    hColumn->nSQLType = SQL_VARCHAR;
    hColumn->pszDescription = 0;
    hColumn->pszName = (char*)strdup( "REMARKS" );
    lstAppend( hTable->hColumns, hColumn );


    /* find table (szTableName) in cache */
    lstFirst( hDatabase->hTables );
    while ( !lstEOL( hDatabase->hTables ) )
    {
        hSourceTable = (HTXTTABLE)lstGet( hDatabase->hTables );

        ppRow		= malloc( sizeof(char*) * hTable->hColumns->nItems );
        ppRow[0]	= (char *)strdup( hDatabase->pszName );
        ppRow[1]	= (char *)strdup( hDatabase->pszName );
        ppRow[2]	= (char *)strdup( hSourceTable->pszName );
        ppRow[3]	= (char *)strdup( "TABLE" );
        ppRow[4]	= (char *)strdup( hSourceTable->pszDescription );

        lstAppend( hTable->hRows, ppRow );        
    
        lstNext( hDatabase->hTables );
    } /* each table */

    /* wrap it up */
    hStmt->nRowsAffected = hTable->hRows->nItems;

    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "SQL_SUCCESS" );
    return SQL_SUCCESS;
}



