#include "driver.h"

/********************
 * ATEMPTS TO LOAD A DATABASE INTO THE CACHE
 ********************/
HTXTDATABASE TXTLoadDatabase( HTXTCACHE hCache, char *pszDatabaseFile )
{
	uid_t			nUserID						= getuid();
	struct passwd	*pPasswd					= NULL;
	char			*pHomeDir					= NULL;
    char            szDatabaseFile[FILENAME_MAX];
    char            szTableFile[FILENAME_MAX];
	char		    szObject[INI_MAX_OBJECT_NAME+1];
	char		    szProperty[INI_MAX_PROPERTY_NAME+1];
	char		    szPropertyValue[INI_MAX_PROPERTY_VALUE+1];
    HTXTDATABASE    hTxtDatabase                = 0;
    HINI            hIniDatabase                = 0;
    HTXTTABLE       hTable                      = 0;
    HTXTCOLUMN      hColumn                     = 0;

    if ( !pszDatabaseFile )
        return 0;

    /* GET DEFAULT PATH (home dir) */	
    pHomeDir	= "/home";		   		 				
    pPasswd		= (struct passwd *)getpwuid( nUserID );	
    if ( pPasswd != NULL )
        if ( (char *)pPasswd->pw_dir != NULL )
            pHomeDir = pPasswd->pw_dir;
    
    if ( *pszDatabaseFile != '/' )
        sprintf( szDatabaseFile, "%s/%s", pHomeDir, pszDatabaseFile );
    else
        sprintf( szDatabaseFile, "%s", pszDatabaseFile );

	if ( iniOpen( &hIniDatabase, szDatabaseFile, ';', '[', ']', '=', 1 ) != INI_SUCCESS )
        return 0;

    /* init a database handle */
    hTxtDatabase    = (HTXTDATABASE)malloc( sizeof(TXTDATABASE) );
    hTxtDatabase->bFixedWidthColumns    = 0;
    hTxtDatabase->cColumnSeperator      = '|';
    lstOpen( &(hTxtDatabase->hTables) );           
    lstSetFreeFunc( hTxtDatabase->hTables, (void(*)(void*))TXTFreeTable );
    hTxtDatabase->pszDescription        = 0;
    hTxtDatabase->pszFileName           = (char *)strdup( szDatabaseFile );
    hTxtDatabase->pszName               = 0;
    hTxtDatabase->pszDefaultPath        = 0;
    lstAppend( hCache->hDatabases, hTxtDatabase );

    /*********************************************
     *
     * load database object definitions
     *
     *********************************************/
    iniObjectFirst( hIniDatabase );
    while ( !iniObjectEOL( hIniDatabase ) )
    {
		*szObject = '\0';
		iniObject( hIniDatabase, szObject );

		if ( strcasecmp( szObject, "DATABASE" ) == 0 )
        {
            /* ONLY ONE DATABASE FOR INI FILE */
            if ( hTxtDatabase->pszName )
                break;

            iniPropertyFirst( hIniDatabase );
            while ( !iniPropertyEOL( hIniDatabase ) )
            {
                *szProperty         = '\0';
                *szPropertyValue    = '\0';
                iniProperty( hIniDatabase, szProperty );
                iniValue( hIniDatabase, szPropertyValue );
                if ( *szPropertyValue != '\0' )
                {
                    if ( strcasecmp( szProperty, "NAME" ) == 0 )
                        hTxtDatabase->pszName = (char*)strdup( szPropertyValue );
                    else if ( strcasecmp( szProperty, "DESCRIPTION" ) == 0 )
                        hTxtDatabase->pszDescription = (char*)strdup( szPropertyValue );
                    else if ( strcasecmp( szProperty, "DEFAULTPATH" ) == 0 )
                        hTxtDatabase->pszDefaultPath = (char*)strdup( szPropertyValue );
                    else if ( strcasecmp( szProperty, "FIXEDWIDTHCOLUMNS" ) == 0 )
                    {
                        if ( *szPropertyValue == '1' || toupper(*szPropertyValue) == 'Y' || toupper(*szPropertyValue) == 'O' || toupper(*szPropertyValue) == 'T' )
                            hTxtDatabase->bFixedWidthColumns = 1;
                    }
                    else if ( strcasecmp( szProperty, "COLUMNSEPERATOR" ) == 0 )
                        hTxtDatabase->cColumnSeperator = *szPropertyValue;
                }
                iniPropertyNext( hIniDatabase );
            }

            /* resolve missing database properties with some default values */
            if ( !hTxtDatabase->pszDescription )
                hTxtDatabase->pszDescription = (char*)strdup( "odbctxt database" );
            if ( !hTxtDatabase->pszName )
                hTxtDatabase->pszName = (char*)strdup( szDatabaseFile );
            if ( !hTxtDatabase->pszDefaultPath )
                hTxtDatabase->pszDefaultPath = (char*)strdup( pHomeDir );

        }
		else if ( strcasecmp( szObject, "TABLE" ) == 0 )
        {
            hTable = (HTXTTABLE)malloc( sizeof(TXTTABLE) );
            lstOpen( &(hTable->hColumns) );
            lstSetFreeFunc( hTable->hColumns, (void(*)(void*))TXTFreeColumn );
            hTable->hDatabase = hTxtDatabase;
            lstOpen( &(hTable->hRows) );
            lstSetFreeFunc( hTable->hRows, (void(*)(void*))TXTFreeRow );
            hTable->pszDescription  = 0;
            hTable->pszName         = 0;
            lstAppend( hTxtDatabase->hTables, hTable );
            hTable->pszFile         = 0;

            iniPropertyFirst( hIniDatabase );
            while ( !iniPropertyEOL( hIniDatabase ) )
            {
                *szProperty         = '\0';
                *szPropertyValue    = '\0';
                iniProperty( hIniDatabase, szProperty );
                iniValue( hIniDatabase, szPropertyValue );
                if ( *szPropertyValue != '\0' )
                {
                    if ( strcasecmp( szProperty, "NAME" ) == 0 )
                        hTable->pszName = (char*)strdup( szPropertyValue );
                    else if ( strcasecmp( szProperty, "DESCRIPTION" ) == 0 )
                        hTable->pszDescription = (char*)strdup( szPropertyValue );
                    else if ( strcasecmp( szProperty, "FILE" ) == 0 )
                        hTable->pszFile = (char*)strdup( szPropertyValue );
                }
                iniPropertyNext( hIniDatabase );
            }

            /* resolve missing table properties */
            if ( !hTable->pszDescription )
                hTable->pszDescription = (char*)strdup( "odbctxt table" );
            if ( !hTable->pszName )
                hTable->pszName = (char*)strdup( "unknown" );
            if ( !hTable->pszFile )
            {
                sprintf( szTableFile, "%s.table", hTable->pszName );
                hTable->pszFile = (char*)strdup( szTableFile );
            }
        }
		else if ( strcasecmp( szObject, "COLUMN" ) == 0 )
        {
            if ( !hTable )
                break;

            hColumn = (HTXTCOLUMN)malloc( sizeof(TXTCOLUMN) );
            hColumn->hTable         = hTable;
            hColumn->nLength        = 255;
            hColumn->nPrecision     = 0;
            hColumn->nSQLType       = SQL_VARCHAR;
            hColumn->pszDescription = 0;
            hColumn->pszName        = 0;
            lstAppend( hTable->hColumns, hColumn );

            iniPropertyFirst( hIniDatabase );
            while ( !iniPropertyEOL( hIniDatabase ) )
            {
                *szProperty         = '\0';
                *szPropertyValue    = '\0';
                iniProperty( hIniDatabase, szProperty );
                iniValue( hIniDatabase, szPropertyValue );
                if ( *szPropertyValue != '\0' )
                {
                    if ( strcasecmp( szProperty, "NAME" ) == 0 )
                        hColumn->pszName = (char*)strdup( szPropertyValue );
                    else if ( strcasecmp( szProperty, "DESCRIPTION" ) == 0 )
                        hColumn->pszDescription = (char*)strdup( szPropertyValue );
                    else if ( strcasecmp( szProperty, "LENGTH" ) == 0 )
                        hColumn->nLength = atoi( szPropertyValue );
                    else if ( strcasecmp( szProperty, "PRECISION" ) == 0 )
                        hColumn->nPrecision = atoi( szPropertyValue );
                    else if ( strcasecmp( szProperty, "TYPE" ) == 0 )
                    {
                        if ( strcasecmp( szPropertyValue, "VARCHAR" ) == 0 )
                            hColumn->nSQLType = SQL_VARCHAR;
                        else if ( strcasecmp( szPropertyValue, "NUMERIC" ) == 0 )
                            hColumn->nSQLType = SQL_NUMERIC;
                        else if ( strcasecmp( szPropertyValue, "DATETIME" ) == 0 )
                            hColumn->nSQLType = SQL_DATETIME;
                    }
                }
                iniPropertyNext( hIniDatabase );
            }

            /* resolve missing table properties */
            if ( !hColumn->pszDescription )
                hColumn->pszDescription = (char*)strdup( "odbctxt column" );
            if ( !hColumn->pszName )
                hColumn->pszName = (char*)strdup( "unknown" );
        }
        iniObjectNext( hIniDatabase );
    }
    iniClose( hIniDatabase );

    /* resolve missing database properties (case where no DATABASE object in INI file) */
    if ( !hTxtDatabase->pszDescription )
        hTxtDatabase->pszDescription = (char*)strdup( "odbctxt database" );
    if ( !hTxtDatabase->pszName )
        hTxtDatabase->pszName = (char*)strdup( szDatabaseFile );
    if ( !hTxtDatabase->pszDefaultPath )
        hTxtDatabase->pszDefaultPath = (char*)strdup( pHomeDir );


    /*********************************************
     *
     * load all table data
     *
     *********************************************/
    lstFirst( hTxtDatabase->hTables );
    while ( !lstEOL( hTxtDatabase->hTables ) )
    {
        TXTLoadTable( (HTXTTABLE)lstGet( hTxtDatabase->hTables ) );
        lstNext( hTxtDatabase->hTables );
    }

    return hTxtDatabase;
}



