/**********************************************
 * Driver.h
 *
 * Description:
 *
 * This is all of the stuff that is common among ALL drivers (but not to the DriverManager).
 *
 * Make sure that your driver specific driverextras.h exists!
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 **********************************************/		
#ifndef _H_DRIVER
#define _H_DRIVER

/*****************************************************************************
 * ODBC VERSION (THAT THIS DRIVER COMPLIES WITH)
 *****************************************************************************/
#define ODBCVER 0x0351 /* this is optimistic to say the least but we are heading to full 3.51 compliance */

#include <log.h>
#include <ini.h>
#include <sqlext.h>
#include <odbcinstext.h>
#include "driverextras.h"

#define SQL_MAX_CURSOR_NAME 100

/*****************************************************************************
 * STATEMENT
 *****************************************************************************/
typedef struct     tDRVSTMT
{
	struct tDRVSTMT	*pPrev;									/* prev struct or null		*/
	struct tDRVSTMT	*pNext;									/* next struct or null		*/
    SQLPOINTER 		hDbc;									/* pointer to DB context    */

    SQLCHAR    		szCursorName[SQL_MAX_CURSOR_NAME];		/* name of cursor           */
    SQLCHAR    		*pszQuery;								/* query string             */
	int				nRowsAffected;							/* is set when SQL Executed	*/
	SQLCHAR     	szSqlMsg[LOG_MSG_MAX];					/* buff to format msgs		*/
	HLOG			hLog;									/* handle to msg logs		*/

    /* TXT DRIVER SPECIFIC */
	HLST	    	hBoundCols;                             /* LST OF HBOUNDCOLUMN      */
    HTXTTABLE       hTable;                                 /* lstOpenCursor resultset  */

} DRVSTMT, *HDRVSTMT;

/*****************************************************************************
 * CONNECTION
 *****************************************************************************/
typedef struct     tDRVDBC
{
	struct tDRVDBC	*pPrev;								/* prev struct or null		*/
	struct tDRVDBC	*pNext;								/* next struct or null		*/
    SQLPOINTER		hEnv; 	                           	/* pointer to ENV structure */
	HDRVSTMT		hFirstStmt;							/* first in list or null	*/
	HDRVSTMT		hLastStmt;							/* last in list or null		*/

	SQLCHAR     	szSqlMsg[LOG_MSG_MAX];				/* buff to format msgs		*/
	HLOG			hLog;								/* handle to msg logs		*/

    int	        	bConnected;                         /* TRUE on open connection  */

    /* TXT DRIVER SPECIFIC */
    HTXTDATABASE    hDatabase;                          /* ref to open database     */
    HLST            hDatabases;                         /* open cursor will ensure  */
                                                        /* hDatabase stays around   */
                                                         
} DRVDBC, *HDRVDBC;

/*****************************************************************************
 * ENVIRONMENT
 *****************************************************************************/
typedef struct     tDRVENV
{
	HDRVDBC		hFirstDbc;							/* first in list or null	*/
	HDRVDBC		hLastDbc;							/* last in list or null		*/

    SQLCHAR     szSqlMsg[LOG_MSG_MAX];				/* buff to format msgs		*/
	HLOG		hLog;								/* handle to msg logs		*/

} DRVENV, *HDRVENV;

/**********************************************
 * BOUNDCOLUMN: One of this for each bound column
 **********************************************/
typedef struct tBOUNDCOLUMN
{
	short			nTargetType;					/* BIND: C DATA TYPE ie SQL_C_CHAR			*/
	char 			*pTargetValue;					/* BIND: POINTER FROM APPLICATION TO COPY TO*/
	long			nTargetValueMax;				/* BIND: MAX SPACE IN pTargetValue			*/
	long			*pnLengthOrIndicator;			/* BIND: TO RETURN LENGTH OR NULL INDICATOR	*/

    /* TXT DRIVER SPECIFIC */
	HTXTCOLUMN		hColumn;						/* BIND: The column details.				*/

} BOUNDCOLUMN, *HBOUNDCOLUMN;


/****************************
 * MAPPED FUNCS
 * Some driver funcs may be called from within the driver (see SQLAllocHandle and SQLFreeHandle). This can 
 * result in code in the Driver Manager being called because the Driver Manager contains the 
 * same functions... and this is not what we want.
 * So we create some similarly named funcs and call them instead.
 ***************************/
SQLRETURN SQLAllocEnv_      ( SQLHENV *phDrvEnv );
SQLRETURN SQLAllocConnect_  ( SQLHENV hDrvEnv, SQLHDBC *phDrvDbc );
SQLRETURN SQLAllocStmt_     ( SQLHDBC hDrvDbc, SQLHSTMT *phDrvStmt );

SQLRETURN SQLFreeEnv_       ( SQLHENV hDrvEnv );
SQLRETURN SQLFreeConnect_   ( SQLHDBC hDrvDbc );
SQLRETURN SQLFreeStmt_      ( SQLHSTMT hDrvStmt, SQLUSMALLINT nOption );

SQLRETURN SQLPrepare_       ( SQLHSTMT hDrvStmt, SQLCHAR *szSqlStr, SQLINTEGER nSqlStrLength );
SQLRETURN SQLExecute_       ( SQLHSTMT  hDrvStmt );
SQLRETURN SQLGetData_       ( SQLHSTMT hDrvStmt, SQLUSMALLINT nCol, SQLSMALLINT nTargetType, SQLPOINTER pTarget, SQLINTEGER nTargetLength, SQLINTEGER *pnLengthOrIndicator );


/****************************
 * FREE FUNCS FOR SQL HANDLES
 ***************************/
SQLRETURN DRVFreeDbcList    ( HDRVENV hDrvEnv );
SQLRETURN DRVFreeStmtList   ( HDRVDBC hDrvDbc );
                            
SQLRETURN DRVFreeEnv        ( HDRVENV hDrvEnv );
SQLRETURN DRVFreeDbc        ( HDRVDBC hDrvDbc );
SQLRETURN DRVFreeStmt       ( HDRVSTMT hDrvStmt );



#endif


