/**********************************************
 * driverextras.h
 *
 * Purpose:
 *
 * To define driver specifc extras such as structs to be included
 * along side the common ODBC includes.
 *
 * In this case; we do not have a DBMS (SQL server or even an API) so
 * we define some data cache structs.
 *
 **********************************************************************
 *
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 **********************************************************************/

#ifndef DRIVEREXTRAS_H
#define DRIVEREXTRAS_H

/**********************************************
 * KEEP IT SIMPLE; PUT ALL DRIVER INCLUDES HERE THEN EACH DRIVER MODULE JUST INCLUDES THIS ONE FILE
 **********************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

#include <lst.h>

/**********************************************
 * DATA CACHE
 * - cache contains all data for each open database
 * - a database is loaded into cache when the 1st ref is created
 * - a database is removed from cache when the last ref is removed
 **********************************************/
typedef struct tTXTCOLUMN
{
    char *  pszName;                            /* name used to id column in this table                             */
    char *  pszDescription;                     /* long name describing purpose of column                           */
    int     nSQLType;                           /* SQL_VARCHAR, SQL_NUMERIC, SQL_DATETIME...                        */
    int     nLength;                            /* length of data as per VARCHAR(nLength) and NUMERIC(nLength)      */
    int     nPrecision;                         /* precision                                                        */
	void *  hTable;								/* handle to table (always base struct in cache)                    */
    int     nCol;                               /* index into row data                                              */
    
} TXTCOLUMN, *HTXTCOLUMN;

typedef struct tTXTTABLE
{
    char *  pszName;                            /* name used to id table in this database                           */
    char *  pszDescription;                     /* long name describing purpose of table                            */
    char *  pszFile;                            /* file name with path else path= TXTDATABASE.pszDefaultPath        */
    HLST    hColumns;                           /* column definitions                                               */
    HLST    hRows;                              /* table data, each HLSTITEM is... char * pRow[]                    */
    void *  hDatabase;                          /* handle to parent database                                        */

} TXTTABLE, *HTXTTABLE;

typedef struct tTXTDATABASE
{
    char *  pszName;                            /* name used to id this database (file name can also be used)       */
    char *  pszDescription;                     /* long name describing the purpose of this database                */
    char *  pszFileName;                        /* name of INI file describing database                             */
    char *  pszDefaultPath;                     /* default path to table files                                      */
    char    cColumnSeperator;                   /* car used to indicate end of column data (if not fixed width)     */
    int     bFixedWidthColumns;                 /* true if we store defined col len regardless of actual col length */
    HLST    hTables;                            /* Tables in this database. This is the 1st and last ref to the     */
                                                /* table data. This means that tables in the cache will not be freed*/
                                                /* until the last ref to the database is gone.                      */

} TXTDATABASE, *HTXTDATABASE;

typedef struct tTXTCACHE
{
    HLST    hDatabases;                         /* list of databases in the cache                                   */

} TXTCACHE, *HTXTCACHE;


extern HTXTCACHE hCache;                        /* entry point to our cache (see _init.c and _fini.c)               */


/****************************
 * LOAD FUNCS FOR DATA CACHE TYPES
 ***************************/
HTXTDATABASE TXTLoadDatabase( HTXTCACHE hCache, char *pszDatabaseFile );
int TXTLoadTable( HTXTTABLE hTable );

/****************************
 * FREE FUNCS FOR DATA CACHE TYPES
 * - these will often be used as lstFreeFunc() callbacks
 ***************************/
void TXTFreeCache           ( HTXTCACHE hCache );
void TXTFreeDatabase        ( HTXTDATABASE hDatabase );
void TXTFreeTable           ( HTXTTABLE hTable );
void TXTFreeColumn          ( HTXTCOLUMN hColumn );
void TXTFreeRow             ( char **ppRow );


#endif


